import { Component, Input, ViewChild, Renderer2, EventEmitter, Output, HostListener, ViewContainerRef } from '@angular/core';
import { TransitionController } from '../transition/transition-controller';
import { Transition, TransitionDirection } from '../transition/transition';
import { KeyCode, parseBooleanAttribute } from '../util/util';
import { ModalControls } from './modal-controls';
import { ModalConfig } from './modal-config';
var SuiModal = (function () {
    /**
     * @param {?} _renderer
     */
    function SuiModal(_renderer) {
        var _this = this;
        this._renderer = _renderer;
        // Initialise with default configuration from `ModalConfig` (to avoid writing defaults twice).
        var config = new ModalConfig();
        this.loadConfig(config);
        // Event emitters for each of the possible modal outcomes.
        this.onApprove = new EventEmitter();
        this.onDeny = new EventEmitter();
        this.onDismiss = new EventEmitter();
        // Initialise controls with actions for the `approve` and `deny` cases.
        this.controls = new ModalControls(function (res) { return _this.dismiss(function () { return _this.onApprove.emit(res); }); }, function (res) { return _this.dismiss(function () { return _this.onDeny.emit(res); }); });
        // Internal variable initialisation.
        this.dimBackground = false;
        this._isClosing = false;
        this.transitionController = new TransitionController(false);
    }
    Object.defineProperty(SuiModal.prototype, "approve", {
        /**
         * @return {?}
         */
        get: function () {
            return this.controls.approve;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiModal.prototype, "deny", {
        /**
         * @return {?}
         */
        get: function () {
            return this.controls.deny;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiModal.prototype, "isFullScreen", {
        /**
         * @return {?}
         */
        get: function () {
            return this._isFullScreen;
        },
        /**
         * @param {?} fullScreen
         * @return {?}
         */
        set: function (fullScreen) {
            this._isFullScreen = parseBooleanAttribute(fullScreen);
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    SuiModal.prototype.ngOnInit = function () {
        var _this = this;
        // Transition the modal to be visible.
        this.transitionController.animate(new Transition(this.transition, this.transitionDuration, TransitionDirection.In));
        // Use a slight delay as the `<sui-dimmer>` cancels the initial transition.
        setTimeout(function () { return _this.dimBackground = true; });
    };
    /**
     * @return {?}
     */
    SuiModal.prototype.ngAfterViewInit = function () {
        // Update margin offset to center modal correctly on-screen.
        var /** @type {?} */ element = (this._modalElement.nativeElement);
        this._renderer.setStyle(element, "margin-top", "-" + element.clientHeight / 2 + "px");
    };
    /**
     * @template V
     * @param {?} config
     * @return {?}
     */
    SuiModal.prototype.loadConfig = function (config) {
        this.isClosable = config.isClosable;
        this.closeResult = config.closeResult;
        this.size = config.size;
        this.isFullScreen = config.isFullScreen;
        this.isBasic = config.isBasic;
        this.transition = config.transition;
        this.transitionDuration = config.transitionDuration;
    };
    /**
     * @param {?=} callback
     * @return {?}
     */
    SuiModal.prototype.dismiss = function (callback) {
        var _this = this;
        if (callback === void 0) { callback = function () { }; }
        // If we aren't currently closing,
        if (!this._isClosing) {
            this._isClosing = true;
            // Transition the modal to be invisible.
            this.dimBackground = false;
            this.transitionController.stopAll();
            this.transitionController.animate(new Transition(this.transition, this.transitionDuration, TransitionDirection.Out, function () {
                // When done, emit a dismiss event, and fire the callback.
                _this.onDismiss.emit();
                callback();
            }));
        }
    };
    /**
     * @return {?}
     */
    SuiModal.prototype.close = function () {
        if (this.isClosable) {
            // If we are allowed to close, fire the deny result with the default value.
            this.deny(this.closeResult);
        }
    };
    /**
     * @param {?} e
     * @return {?}
     */
    SuiModal.prototype.onKeyup = function (e) {
        if (e.keyCode == KeyCode.Escape) {
            // Close automatically covers case of `!isClosable`, so check not needed.
            this.close();
        }
    };
    return SuiModal;
}());
export { SuiModal };
SuiModal.decorators = [
    { type: Component, args: [{
                selector: 'sui-modal',
                template: "\n<!-- Page dimmer for modal background. -->\n<sui-dimmer class=\"page\" [(isDimmed)]=\"dimBackground\" [isClickable]=\"false\" [transitionDuration]=\"transitionDuration\" (click)=\"close()\"></sui-dimmer>\n<!-- Modal component, with transition component attached -->\n<div class=\"ui modal {{ size }}\"\n     [suiTransition]=\"transitionController\"\n     [class.active]=\"transitionController?.isVisible\"\n     [class.fullscreen]=\"isFullScreen\"\n     [class.basic]=\"isBasic\"\n     #modal>\n\n    <!-- Configurable close icon -->\n    <i class=\"close icon\" *ngIf=\"isClosable\" (click)=\"close()\"></i>\n    <!-- <ng-content> so that <sui-modal> can be used as a normal component. -->\n    <ng-content></ng-content>\n    <!-- @ViewChild reference so we can insert elements beside this div. -->\n    <div #templateSibling></div>\n</div>\n"
            },] },
];
/**
 * @nocollapse
 */
SuiModal.ctorParameters = function () { return [
    { type: Renderer2, },
]; };
SuiModal.propDecorators = {
    'isClosable': [{ type: Input },],
    'closeResult': [{ type: Input },],
    'onApprove': [{ type: Output, args: ['approved',] },],
    'onDeny': [{ type: Output, args: ['denied',] },],
    'onDismiss': [{ type: Output, args: ['dismissed',] },],
    '_modalElement': [{ type: ViewChild, args: ['modal',] },],
    'size': [{ type: Input },],
    'isFullScreen': [{ type: Input },],
    'isBasic': [{ type: Input },],
    'transition': [{ type: Input },],
    'transitionDuration': [{ type: Input },],
    'templateSibling': [{ type: ViewChild, args: ['templateSibling', { read: ViewContainerRef },] },],
    'onKeyup': [{ type: HostListener, args: ["document:keyup", ["$event"],] },],
};
function SuiModal_tsickle_Closure_declarations() {
    /** @type {?} */
    SuiModal.decorators;
    /**
     * @nocollapse
     * @type {?}
     */
    SuiModal.ctorParameters;
    /** @type {?} */
    SuiModal.propDecorators;
    /** @type {?} */
    SuiModal.prototype.isClosable;
    /** @type {?} */
    SuiModal.prototype.closeResult;
    /** @type {?} */
    SuiModal.prototype.controls;
    /** @type {?} */
    SuiModal.prototype.onApprove;
    /** @type {?} */
    SuiModal.prototype.onDeny;
    /** @type {?} */
    SuiModal.prototype.onDismiss;
    /** @type {?} */
    SuiModal.prototype._modalElement;
    /** @type {?} */
    SuiModal.prototype.size;
    /** @type {?} */
    SuiModal.prototype._isFullScreen;
    /** @type {?} */
    SuiModal.prototype.isBasic;
    /** @type {?} */
    SuiModal.prototype.transitionController;
    /** @type {?} */
    SuiModal.prototype.transition;
    /** @type {?} */
    SuiModal.prototype.transitionDuration;
    /** @type {?} */
    SuiModal.prototype.dimBackground;
    /** @type {?} */
    SuiModal.prototype._isClosing;
    /** @type {?} */
    SuiModal.prototype.templateSibling;
    /** @type {?} */
    SuiModal.prototype._renderer;
}
//# sourceMappingURL=modal.js.map