import { Component, ViewChild, ViewContainerRef, ElementRef, EventEmitter, HostListener } from '@angular/core';
import { Transition, TransitionDirection } from '../transition/transition';
import { TransitionController } from '../transition/transition-controller';
import { PositioningService } from '../util/positioning.service';
var SuiPopup = (function () {
    /**
     * @param {?} elementRef
     */
    function SuiPopup(elementRef) {
        this.elementRef = elementRef;
        this.transitionController = new TransitionController(false);
        this._isOpen = false;
        this.onClose = new EventEmitter();
    }
    Object.defineProperty(SuiPopup.prototype, "isOpen", {
        /**
         * @return {?}
         */
        get: function () {
            return this._isOpen;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiPopup.prototype, "anchor", {
        /**
         * @param {?} anchor
         * @return {?}
         */
        set: function (anchor) {
            // Whenever the anchor is set (which is when the popup is created), recreate the positioning service with the appropriate options.
            this._positioningService = new PositioningService(anchor, this._container.element, this.config.placement, ".dynamic.arrow");
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiPopup.prototype, "direction", {
        /**
         * @return {?}
         */
        get: function () {
            if (this._positioningService) {
                return this._positioningService.placement.split(" ").shift();
            }
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @return {?}
     */
    SuiPopup.prototype.open = function () {
        var _this = this;
        // Only attempt to open if currently closed.
        if (!this.isOpen) {
            // Cancel the closing timer.
            clearTimeout(this._closingTimeout);
            // Cancel all other transitions, and initiate the opening transition.
            this.transitionController.stopAll();
            this.transitionController.animate(new Transition(this.config.transition, this.config.transitionDuration, TransitionDirection.In));
            // Refresh the popup position after a brief delay to allow for browser processing time.
            setTimeout(function () { return _this._positioningService.update(); });
            // Finally, set the popup to be open.
            this._isOpen = true;
        }
    };
    /**
     * @return {?}
     */
    SuiPopup.prototype.toggle = function () {
        if (!this.isOpen) {
            return this.open();
        }
        return this.close();
    };
    /**
     * @return {?}
     */
    SuiPopup.prototype.close = function () {
        var _this = this;
        // Only attempt to close if currently open.
        if (this.isOpen) {
            // Cancel all other transitions, and initiate the closing transition.
            this.transitionController.stopAll();
            this.transitionController.animate(new Transition(this.config.transition, this.config.transitionDuration, TransitionDirection.Out));
            // Cancel the closing timer.
            clearTimeout(this._closingTimeout);
            // Start the closing timer, that fires the `onClose` event after the transition duration number of milliseconds.
            this._closingTimeout = window.setTimeout(function () { return _this.onClose.emit(); }, this.config.transitionDuration);
            // Finally, set the popup to be closed.
            this._isOpen = false;
        }
    };
    /**
     * @param {?} event
     * @return {?}
     */
    SuiPopup.prototype.onClick = function (event) {
        // Makes sense here, as the popup shouldn't be attached to any DOM element.
        event.stopPropagation();
    };
    return SuiPopup;
}());
export { SuiPopup };
SuiPopup.decorators = [
    { type: Component, args: [{
                selector: 'sui-popup',
                template: "\n<div class=\"ui popup\" [class.inverted]=\"config.isInverted\" [class.basic]=\"config.isBasic\" [suiTransition]=\"transitionController\" [attr.direction]=\"direction\" #container>\n    <ng-container *ngIf=\"!config.template\">\n        <div class=\"header\" *ngIf=\"config.header\">{{ config.header }}</div>\n        <div class=\"content\">{{ config.text }}</div>\n    </ng-container>\n    <div #templateSibling></div>\n    <sui-popup-arrow *ngIf=\"!config.isBasic\" [placement]=\"_positioningService.placement\" [inverted]=\"config.isInverted\"></sui-popup-arrow>\n</div>\n",
                styles: ["\n.ui.popup {\n    /* Autofit popup to the contents. */\n    right: auto;\n}\n\n.ui.animating.popup {\n    /* When the popup is animating, it may not initially be in the correct position.\n       This fires a mouse event, causing the anchor's mouseleave to fire - making the popup flicker.\n       Setting pointer-events to none while animating fixes this bug. */\n    pointer-events: none;\n}\n\n.ui.popup::before {\n    /* Hide the Semantic UI CSS arrow. */\n    display: none;\n}\n\n/* Offset popup by 0.75em above and below when placed 'vertically'. */\n.ui.popup[direction=\"top\"],\n.ui.popup[direction=\"bottom\"] {\n    margin-top: 0.75em;\n    margin-bottom: 0.75em;\n}\n\n/* Offset popup by 0.75em either side when placed 'horizontally'. */\n.ui.popup[direction=\"left\"],\n.ui.popup[direction=\"right\"] {\n    margin-left: 0.75em;\n    margin-right: 0.75em;\n}\n"]
            },] },
];
/**
 * @nocollapse
 */
SuiPopup.ctorParameters = function () { return [
    { type: ElementRef, },
]; };
SuiPopup.propDecorators = {
    '_container': [{ type: ViewChild, args: ['container', { read: ViewContainerRef },] },],
    'templateSibling': [{ type: ViewChild, args: ['templateSibling', { read: ViewContainerRef },] },],
    'onClick': [{ type: HostListener, args: ["click", ["$event"],] },],
};
function SuiPopup_tsickle_Closure_declarations() {
    /** @type {?} */
    SuiPopup.decorators;
    /**
     * @nocollapse
     * @type {?}
     */
    SuiPopup.ctorParameters;
    /** @type {?} */
    SuiPopup.propDecorators;
    /** @type {?} */
    SuiPopup.prototype.config;
    /** @type {?} */
    SuiPopup.prototype.transitionController;
    /** @type {?} */
    SuiPopup.prototype._positioningService;
    /** @type {?} */
    SuiPopup.prototype._isOpen;
    /** @type {?} */
    SuiPopup.prototype._closingTimeout;
    /** @type {?} */
    SuiPopup.prototype.onClose;
    /** @type {?} */
    SuiPopup.prototype._container;
    /** @type {?} */
    SuiPopup.prototype.templateSibling;
    /** @type {?} */
    SuiPopup.prototype.elementRef;
}
//# sourceMappingURL=popup.js.map